/**
 * @file
 * @brief Hardware configuration
 *
 * Configuration:
 * - center aligned PWM, 10 kHz
 * - ADC being automatically started (by hardware) on every PWM period
 * - ADC complete interrupt executes X2C Update function
 * - LED 'L' (digital output #13) blink frequency can be controlled by X2C block 'Frequency'
 * - UART configuration using 115200/8/N/1 for X2C communication
 *
 * Uses Atmel ASF functions.
 */
/*
 * $LastChangedRevision: 1886 $
 * $LastChangedDate:: 2020-04-23 22:09:26 +0200#$
 *
 * Copyright (c) 2016, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef HARDWARE_H
#define HARDWARE_H

#include "Target.h"

#define resetWatchdog() do { \
	; \
} while (0)

#define BLINKY_LED_ON (gpio_set_pin_high(PIO_PB27_IDX))
#define BLINKY_LED_OFF (gpio_set_pin_low(PIO_PB27_IDX))

/* public prototypes */
void initHardware(void);
void initSerial(tSerial* serial);

#endif /* HARDWARE_H */
